"""Combine cells into rows."""

from terminaltables.width_and_alignment import visible_width


def combine(line, left, intersect, right):
    """Zip borders between items in `line`.

    e.g. ('l', '1', 'c', '2', 'c', '3', 'r')

    :param iter line: List to iterate.
    :param left: Left border.
    :param intersect: Column separator.
    :param right: Right border.

    :return: Yields combined objects.
    """
    # Yield left border.
    if left:
        yield left

    # Yield items with intersect characters.
    if intersect:
        try:
            for j, i in enumerate(line, start=-len(line) + 1):
                yield i
                if j:
                    yield intersect
        except TypeError:  # Generator.
            try:
                item = next(line)
            except StopIteration:  # Was empty all along.
                pass
            else:
                while True:
                    yield item
                    try:
                        peek = next(line)
                    except StopIteration:
                        break
                    yield intersect
                    item = peek
    else:
        for i in line:
            yield i

    # Yield right border.
    if right:
        yield right


def build_border(outer_widths, horizontal, left, intersect, right, title=None):
    """Build the top/bottom/middle row. Optionally embed the table title within the border.

    Title is hidden if it doesn't fit between the left/right characters/edges.

    Example return value:
    ('<', '-----', '+', '------', '+', '-------', '>')
    ('<', 'My Table', '----', '+', '------->')

    :param iter outer_widths: List of widths (with padding) for each column.
    :param str horizontal: Character to stretch across each column.
    :param str left: Left border.
    :param str intersect: Column separator.
    :param str right: Right border.
    :param title: Overlay the title on the border between the left and right characters.

    :return: Returns a generator of strings representing a border.
    :rtype: iter
    """
    length = 0

    # Hide title if it doesn't fit.
    if title is not None and outer_widths:
        try:
            length = visible_width(title)
        except TypeError:
            title = str(title)
            length = visible_width(title)
        if length > sum(outer_widths) + len(intersect) * (len(outer_widths) - 1):
            title = None

    # Handle no title.
    if title is None or not outer_widths or not horizontal:
        return combine((horizontal * c for c in outer_widths), left, intersect, right)

    # Handle title fitting in the first column.
    if length == outer_widths[0]:
        return combine([title] + [horizontal * c for c in outer_widths[1:]], left, intersect, right)
    if length < outer_widths[0]:
        columns = [title + horizontal * (outer_widths[0] - length)] + [horizontal * c for c in outer_widths[1:]]
        return combine(columns, left, intersect, right)

    # Handle wide titles/narrow columns.
    columns_and_intersects = [title]
    for width in combine(outer_widths, None, bool(intersect), None):
        # If title is taken care of.
        if length < 1:
            columns_and_intersects.append(intersect if width is True else horizontal * width)
        # If title's last character overrides an intersect character.
        elif width is True and length == 1:
            length = 0
        # If this is an intersect character that is overridden by the title.
        elif width is True:
            length -= 1
        # If title's last character is within a column.
        elif width >= length:
            columns_and_intersects[0] += horizontal * (width - length)  # Append horizontal chars to title.
            length = 0
        # If remainder of title won't fit in a column.
        else:
            length -= width

    return combine(columns_and_intersects, left, None, right)


def build_row(row, left, center, right):
    """Combine single or multi-lined cells into a single row of list of lists including borders.

    Row must already be padded and extended so each cell has the same number of lines.

    Example return value:
    [
        ['>', 'Left ', '|', 'Center', '|', 'Right', '<'],
        ['>', 'Cell1', '|', 'Cell2 ', '|', 'Cell3', '<'],
    ]

    :param iter row: List of cells for one row.
    :param str left: Left border.
    :param str center: Column separator.
    :param str right: Right border.

    :return: Yields other generators that yield strings.
    :rtype: iter
    """
    if not row or not row[0]:
        yield combine((), left, center, right)
        return
    for row_index in range(len(row[0])):
        yield combine((c[row_index] for c in row), left, center, right)


def flatten(table):
    """Flatten table data into a single string with newlines.

    :param iter table: Padded and bordered table data.

    :return: Joined rows/cells.
    :rtype: str
    """
    return '\n'.join(''.join(r) for r in table)



"""Generate simple tables in terminals from a nested list of strings.

Use SingleTable or DoubleTable instead of AsciiTable for box-drawing characters.

https://github.com/Robpol86/terminaltables
https://pypi.python.org/pypi/terminaltables
"""

from terminaltables.ascii_table import AsciiTable  # noqa
from terminaltables.github_table import GithubFlavoredMarkdownTable  # noqa
from terminaltables.other_tables import DoubleTable  # noqa
from terminaltables.other_tables import SingleTable  # noqa

__author__ = '@Robpol86'
__license__ = 'MIT'
__version__ = '3.0.0'



"""Base table class. Define just the bare minimum to build tables."""

from terminaltables.build import build_border, build_row, flatten
from terminaltables.width_and_alignment import align_and_pad_cell, max_dimensions


class BaseTable(object):
    """Base table class.

    :ivar iter table_data: List (empty or list of lists of strings) representing the table.
    :ivar str title: Optional title to show within the top border of the table.
    :ivar bool inner_column_border: Separates columns.
    :ivar bool inner_footing_row_border: Show a border before the last row.
    :ivar bool inner_heading_row_border: Show a border before the first row.
    :ivar bool inner_row_border: Show a border in between every row.
    :ivar bool outer_border: Show the top, left, right, and bottom border.
    :ivar dict justify_columns: Horizontal justification. Keys are column indexes (int). Values are right/left/center.
    :ivar int padding_left: Number of spaces to pad on the left side of every cell.
    :ivar int padding_right: Number of spaces to pad on the right side of every cell.
    """

    CHAR_F_INNER_HORIZONTAL = '-'
    CHAR_F_INNER_INTERSECT = '+'
    CHAR_F_INNER_VERTICAL = '|'
    CHAR_F_OUTER_LEFT_INTERSECT = '+'
    CHAR_F_OUTER_LEFT_VERTICAL = '|'
    CHAR_F_OUTER_RIGHT_INTERSECT = '+'
    CHAR_F_OUTER_RIGHT_VERTICAL = '|'
    CHAR_H_INNER_HORIZONTAL = '-'
    CHAR_H_INNER_INTERSECT = '+'
    CHAR_H_INNER_VERTICAL = '|'
    CHAR_H_OUTER_LEFT_INTERSECT = '+'
    CHAR_H_OUTER_LEFT_VERTICAL = '|'
    CHAR_H_OUTER_RIGHT_INTERSECT = '+'
    CHAR_H_OUTER_RIGHT_VERTICAL = '|'
    CHAR_INNER_HORIZONTAL = '-'
    CHAR_INNER_INTERSECT = '+'
    CHAR_INNER_VERTICAL = '|'
    CHAR_OUTER_BOTTOM_HORIZONTAL = '-'
    CHAR_OUTER_BOTTOM_INTERSECT = '+'
    CHAR_OUTER_BOTTOM_LEFT = '+'
    CHAR_OUTER_BOTTOM_RIGHT = '+'
    CHAR_OUTER_LEFT_INTERSECT = '+'
    CHAR_OUTER_LEFT_VERTICAL = '|'
    CHAR_OUTER_RIGHT_INTERSECT = '+'
    CHAR_OUTER_RIGHT_VERTICAL = '|'
    CHAR_OUTER_TOP_HORIZONTAL = '-'
    CHAR_OUTER_TOP_INTERSECT = '+'
    CHAR_OUTER_TOP_LEFT = '+'
    CHAR_OUTER_TOP_RIGHT = '+'

    def __init__(self, table_data, title=None):
        """Constructor.

        :param iter table_data: List (empty or list of lists of strings) representing the table.
        :param title: Optional title to show within the top border of the table.
        """
        self.table_data = table_data
        self.title = title

        self.inner_column_border = True
        self.inner_footing_row_border = False
        self.inner_heading_row_border = True
        self.inner_row_border = False
        self.outer_border = True

        self.justify_columns = dict()  # {0: 'right', 1: 'left', 2: 'center'}
        self.padding_left = 1
        self.padding_right = 1

    def horizontal_border(self, style, outer_widths):
        """Build any kind of horizontal border for the table.

        :param str style: Type of border to return.
        :param iter outer_widths: List of widths (with padding) for each column.

        :return: Prepared border as a tuple of strings.
        :rtype: tuple
        """
        if style == 'top':
            horizontal = self.CHAR_OUTER_TOP_HORIZONTAL
            left = self.CHAR_OUTER_TOP_LEFT
            intersect = self.CHAR_OUTER_TOP_INTERSECT if self.inner_column_border else ''
            right = self.CHAR_OUTER_TOP_RIGHT
            title = self.title
        elif style == 'bottom':
            horizontal = self.CHAR_OUTER_BOTTOM_HORIZONTAL
            left = self.CHAR_OUTER_BOTTOM_LEFT
            intersect = self.CHAR_OUTER_BOTTOM_INTERSECT if self.inner_column_border else ''
            right = self.CHAR_OUTER_BOTTOM_RIGHT
            title = None
        elif style == 'heading':
            horizontal = self.CHAR_H_INNER_HORIZONTAL
            left = self.CHAR_H_OUTER_LEFT_INTERSECT if self.outer_border else ''
            intersect = self.CHAR_H_INNER_INTERSECT if self.inner_column_border else ''
            right = self.CHAR_H_OUTER_RIGHT_INTERSECT if self.outer_border else ''
            title = None
        elif style == 'footing':
            horizontal = self.CHAR_F_INNER_HORIZONTAL
            left = self.CHAR_F_OUTER_LEFT_INTERSECT if self.outer_border else ''
            intersect = self.CHAR_F_INNER_INTERSECT if self.inner_column_border else ''
            right = self.CHAR_F_OUTER_RIGHT_INTERSECT if self.outer_border else ''
            title = None
        else:
            horizontal = self.CHAR_INNER_HORIZONTAL
            left = self.CHAR_OUTER_LEFT_INTERSECT if self.outer_border else ''
            intersect = self.CHAR_INNER_INTERSECT if self.inner_column_border else ''
            right = self.CHAR_OUTER_RIGHT_INTERSECT if self.outer_border else ''
            title = None
        return build_border(outer_widths, horizontal, left, intersect, right, title)

    def gen_row_lines(self, row, style, inner_widths, height):
        r"""Combine cells in row and group them into lines with vertical borders.

        Caller is expected to pass yielded lines to ''.join() to combine them into a printable line. Caller must append
        newline character to the end of joined line.

        In:
        ['Row One Column One', 'Two', 'Three']
        Out:
        [
            ('|', ' Row One Column One ', '|', ' Two ', '|', ' Three ', '|'),
        ]

        In:
        ['Row One\nColumn One', 'Two', 'Three'],
        Out:
        [
            ('|', ' Row One    ', '|', ' Two ', '|', ' Three ', '|'),
            ('|', ' Column One ', '|', '     ', '|', '       ', '|'),
        ]

        :param iter row: One row in the table. List of cells.
        :param str style: Type of border characters to use.
        :param iter inner_widths: List of widths (no padding) for each column.
        :param int height: Inner height (no padding) (number of lines) to expand row to.

        :return: Yields lines split into components in a list. Caller must ''.join() line.
        """
        cells_in_row = list()

        # Resize row if it doesn't have enough cells.
        if len(row) != len(inner_widths):
            row = row + [''] * (len(inner_widths) - len(row))

        # Pad and align each cell. Split each cell into lines to support multi-line cells.
        for i, cell in enumerate(row):
            align = (self.justify_columns.get(i),)
            inner_dimensions = (inner_widths[i], height)
            padding = (self.padding_left, self.padding_right, 0, 0)
            cells_in_row.append(align_and_pad_cell(cell, align, inner_dimensions, padding))

        # Determine border characters.
        if style == 'heading':
            left = self.CHAR_H_OUTER_LEFT_VERTICAL if self.outer_border else ''
            center = self.CHAR_H_INNER_VERTICAL if self.inner_column_border else ''
            right = self.CHAR_H_OUTER_RIGHT_VERTICAL if self.outer_border else ''
        elif style == 'footing':
            left = self.CHAR_F_OUTER_LEFT_VERTICAL if self.outer_border else ''
            center = self.CHAR_F_INNER_VERTICAL if self.inner_column_border else ''
            right = self.CHAR_F_OUTER_RIGHT_VERTICAL if self.outer_border else ''
        else:
            left = self.CHAR_OUTER_LEFT_VERTICAL if self.outer_border else ''
            center = self.CHAR_INNER_VERTICAL if self.inner_column_border else ''
            right = self.CHAR_OUTER_RIGHT_VERTICAL if self.outer_border else ''

        # Yield each line.
        for line in build_row(cells_in_row, left, center, right):
            yield line

    def gen_table(self, inner_widths, inner_heights, outer_widths):
        """Combine everything and yield every line of the entire table with borders.

        :param iter inner_widths: List of widths (no padding) for each column.
        :param iter inner_heights: List of heights (no padding) for each row.
        :param iter outer_widths: List of widths (with padding) for each column.
        :return:
        """
        # Yield top border.
        if self.outer_border:
            yield self.horizontal_border('top', outer_widths)

        # Yield table body.
        row_count = len(self.table_data)
        last_row_index, before_last_row_index = row_count - 1, row_count - 2
        for i, row in enumerate(self.table_data):
            # Yield the row line by line (e.g. multi-line rows).
            if self.inner_heading_row_border and i == 0:
                style = 'heading'
            elif self.inner_footing_row_border and i == last_row_index:
                style = 'footing'
            else:
                style = 'row'
            for line in self.gen_row_lines(row, style, inner_widths, inner_heights[i]):
                yield line
            # If this is the last row then break. No separator needed.
            if i == last_row_index:
                break
            # Yield heading separator.
            if self.inner_heading_row_border and i == 0:
                yield self.horizontal_border('heading', outer_widths)
            # Yield footing separator.
            elif self.inner_footing_row_border and i == before_last_row_index:
                yield self.horizontal_border('footing', outer_widths)
            # Yield row separator.
            elif self.inner_row_border:
                yield self.horizontal_border('row', outer_widths)

        # Yield bottom border.
        if self.outer_border:
            yield self.horizontal_border('bottom', outer_widths)

    @property
    def table(self):
        """Return a large string of the entire table ready to be printed to the terminal."""
        dimensions = max_dimensions(self.table_data, self.padding_left, self.padding_right)[:3]
        return flatten(self.gen_table(*dimensions))



"""GithubFlavoredMarkdownTable class."""

from terminaltables.ascii_table import AsciiTable
from terminaltables.build import combine


class GithubFlavoredMarkdownTable(AsciiTable):
    """Github flavored markdown table.

    https://help.github.com/articles/github-flavored-markdown/#tables

    :ivar iter table_data: List (empty or list of lists of strings) representing the table.
    :ivar dict justify_columns: Horizontal justification. Keys are column indexes (int). Values are right/left/center.
    """

    def __init__(self, table_data):
        """Constructor.

        :param iter table_data: List (empty or list of lists of strings) representing the table.
        """
        # Github flavored markdown table won't support title.
        super(GithubFlavoredMarkdownTable, self).__init__(table_data)

    def horizontal_border(self, _, outer_widths):
        """Handle the GitHub heading border.

        E.g.:
        |:---|:---:|---:|----|

        :param _: Unused.
        :param iter outer_widths: List of widths (with padding) for each column.

        :return: Prepared border strings in a generator.
        :rtype: iter
        """
        horizontal = str(self.CHAR_INNER_HORIZONTAL)
        left = self.CHAR_OUTER_LEFT_VERTICAL
        intersect = self.CHAR_INNER_VERTICAL
        right = self.CHAR_OUTER_RIGHT_VERTICAL

        columns = list()
        for i, width in enumerate(outer_widths):
            justify = self.justify_columns.get(i)
            width = max(3, width)  # Width should be at least 3 so justification can be applied.
            if justify == 'left':
                columns.append(':' + horizontal * (width - 1))
            elif justify == 'right':
                columns.append(horizontal * (width - 1) + ':')
            elif justify == 'center':
                columns.append(':' + horizontal * (width - 2) + ':')
            else:
                columns.append(horizontal * width)

        return combine(columns, left, intersect, right)

    def gen_table(self, inner_widths, inner_heights, outer_widths):
        """Combine everything and yield every line of the entire table with borders.

        :param iter inner_widths: List of widths (no padding) for each column.
        :param iter inner_heights: List of heights (no padding) for each row.
        :param iter outer_widths: List of widths (with padding) for each column.
        :return:
        """
        for i, row in enumerate(self.table_data):
            # Yield the row line by line (e.g. multi-line rows).
            for line in self.gen_row_lines(row, 'row', inner_widths, inner_heights[i]):
                yield line
            # Yield heading separator.
            if i == 0:
                yield self.horizontal_border(None, outer_widths)



#!/usr/bin/env python
"""Setup script for the project."""

from __future__ import print_function

import codecs
import os

from setuptools import setup


def readme():
    """Try to read README.rst or return empty string if failed.

    :return: File contents.
    :rtype: str
    """
    path = os.path.realpath(os.path.join(os.path.dirname(__file__), 'README.rst'))
    handle = None
    try:
        handle = codecs.open(path, encoding='utf-8')
        return handle.read(131072)
    except IOError:
        return ''
    finally:
        getattr(handle, 'close', lambda: None)()


setup(
    author='@Robpol86',
    author_email='robpol86@gmail.com',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Environment :: MacOS X',
        'Environment :: Win32 (MS Windows)',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries',
        'Topic :: Terminals',
        'Topic :: Text Processing :: Markup',
    ],
    description='Generate simple tables in terminals from a nested list of strings.',
    install_requires=[],
    keywords='Shell Bash ANSI ASCII terminal tables',
    license='MIT',
    long_description=readme(),
    name='terminaltables',
    packages=['terminaltables'],
    url='https://github.com/Robpol86/terminaltables',
    version='3.0.0',
    zip_safe=True,
)



"""Additional simple tables defined here."""

from terminaltables.ascii_table import AsciiTable
from terminaltables.terminal_io import IS_WINDOWS


class UnixTable(AsciiTable):
    """Draw a table using box-drawing characters on Unix platforms. Table borders won't have any gaps between lines.

    Similar to the tables shown on PC BIOS boot messages, but not double-lined.
    """

    CHAR_F_INNER_HORIZONTAL = '\033(0\x71\033(B'
    CHAR_F_INNER_INTERSECT = '\033(0\x6e\033(B'
    CHAR_F_INNER_VERTICAL = '\033(0\x78\033(B'
    CHAR_F_OUTER_LEFT_INTERSECT = '\033(0\x74\033(B'
    CHAR_F_OUTER_LEFT_VERTICAL = '\033(0\x78\033(B'
    CHAR_F_OUTER_RIGHT_INTERSECT = '\033(0\x75\033(B'
    CHAR_F_OUTER_RIGHT_VERTICAL = '\033(0\x78\033(B'
    CHAR_H_INNER_HORIZONTAL = '\033(0\x71\033(B'
    CHAR_H_INNER_INTERSECT = '\033(0\x6e\033(B'
    CHAR_H_INNER_VERTICAL = '\033(0\x78\033(B'
    CHAR_H_OUTER_LEFT_INTERSECT = '\033(0\x74\033(B'
    CHAR_H_OUTER_LEFT_VERTICAL = '\033(0\x78\033(B'
    CHAR_H_OUTER_RIGHT_INTERSECT = '\033(0\x75\033(B'
    CHAR_H_OUTER_RIGHT_VERTICAL = '\033(0\x78\033(B'
    CHAR_INNER_HORIZONTAL = '\033(0\x71\033(B'
    CHAR_INNER_INTERSECT = '\033(0\x6e\033(B'
    CHAR_INNER_VERTICAL = '\033(0\x78\033(B'
    CHAR_OUTER_BOTTOM_HORIZONTAL = '\033(0\x71\033(B'
    CHAR_OUTER_BOTTOM_INTERSECT = '\033(0\x76\033(B'
    CHAR_OUTER_BOTTOM_LEFT = '\033(0\x6d\033(B'
    CHAR_OUTER_BOTTOM_RIGHT = '\033(0\x6a\033(B'
    CHAR_OUTER_LEFT_INTERSECT = '\033(0\x74\033(B'
    CHAR_OUTER_LEFT_VERTICAL = '\033(0\x78\033(B'
    CHAR_OUTER_RIGHT_INTERSECT = '\033(0\x75\033(B'
    CHAR_OUTER_RIGHT_VERTICAL = '\033(0\x78\033(B'
    CHAR_OUTER_TOP_HORIZONTAL = '\033(0\x71\033(B'
    CHAR_OUTER_TOP_INTERSECT = '\033(0\x77\033(B'
    CHAR_OUTER_TOP_LEFT = '\033(0\x6c\033(B'
    CHAR_OUTER_TOP_RIGHT = '\033(0\x6b\033(B'

    @property
    def table(self):
        """Return a large string of the entire table ready to be printed to the terminal."""
        ascii_table = super(UnixTable, self).table
        optimized = ascii_table.replace('\033(B\033(0', '')
        return optimized


class WindowsTable(AsciiTable):
    """Draw a table using box-drawing characters on Windows platforms. This uses Code Page 437. Single-line borders.

    From: http://en.wikipedia.org/wiki/Code_page_437#Characters
    """

    CHAR_F_INNER_HORIZONTAL = b'\xc4'.decode('ibm437')
    CHAR_F_INNER_INTERSECT = b'\xc5'.decode('ibm437')
    CHAR_F_INNER_VERTICAL = b'\xb3'.decode('ibm437')
    CHAR_F_OUTER_LEFT_INTERSECT = b'\xc3'.decode('ibm437')
    CHAR_F_OUTER_LEFT_VERTICAL = b'\xb3'.decode('ibm437')
    CHAR_F_OUTER_RIGHT_INTERSECT = b'\xb4'.decode('ibm437')
    CHAR_F_OUTER_RIGHT_VERTICAL = b'\xb3'.decode('ibm437')
    CHAR_H_INNER_HORIZONTAL = b'\xc4'.decode('ibm437')
    CHAR_H_INNER_INTERSECT = b'\xc5'.decode('ibm437')
    CHAR_H_INNER_VERTICAL = b'\xb3'.decode('ibm437')
    CHAR_H_OUTER_LEFT_INTERSECT = b'\xc3'.decode('ibm437')
    CHAR_H_OUTER_LEFT_VERTICAL = b'\xb3'.decode('ibm437')
    CHAR_H_OUTER_RIGHT_INTERSECT = b'\xb4'.decode('ibm437')
    CHAR_H_OUTER_RIGHT_VERTICAL = b'\xb3'.decode('ibm437')
    CHAR_INNER_HORIZONTAL = b'\xc4'.decode('ibm437')
    CHAR_INNER_INTERSECT = b'\xc5'.decode('ibm437')
    CHAR_INNER_VERTICAL = b'\xb3'.decode('ibm437')
    CHAR_OUTER_BOTTOM_HORIZONTAL = b'\xc4'.decode('ibm437')
    CHAR_OUTER_BOTTOM_INTERSECT = b'\xc1'.decode('ibm437')
    CHAR_OUTER_BOTTOM_LEFT = b'\xc0'.decode('ibm437')
    CHAR_OUTER_BOTTOM_RIGHT = b'\xd9'.decode('ibm437')
    CHAR_OUTER_LEFT_INTERSECT = b'\xc3'.decode('ibm437')
    CHAR_OUTER_LEFT_VERTICAL = b'\xb3'.decode('ibm437')
    CHAR_OUTER_RIGHT_INTERSECT = b'\xb4'.decode('ibm437')
    CHAR_OUTER_RIGHT_VERTICAL = b'\xb3'.decode('ibm437')
    CHAR_OUTER_TOP_HORIZONTAL = b'\xc4'.decode('ibm437')
    CHAR_OUTER_TOP_INTERSECT = b'\xc2'.decode('ibm437')
    CHAR_OUTER_TOP_LEFT = b'\xda'.decode('ibm437')
    CHAR_OUTER_TOP_RIGHT = b'\xbf'.decode('ibm437')


class WindowsTableDouble(AsciiTable):
    """Draw a table using box-drawing characters on Windows platforms. This uses Code Page 437. Double-line borders."""

    CHAR_F_INNER_HORIZONTAL = b'\xcd'.decode('ibm437')
    CHAR_F_INNER_INTERSECT = b'\xce'.decode('ibm437')
    CHAR_F_INNER_VERTICAL = b'\xba'.decode('ibm437')
    CHAR_F_OUTER_LEFT_INTERSECT = b'\xcc'.decode('ibm437')
    CHAR_F_OUTER_LEFT_VERTICAL = b'\xba'.decode('ibm437')
    CHAR_F_OUTER_RIGHT_INTERSECT = b'\xb9'.decode('ibm437')
    CHAR_F_OUTER_RIGHT_VERTICAL = b'\xba'.decode('ibm437')
    CHAR_H_INNER_HORIZONTAL = b'\xcd'.decode('ibm437')
    CHAR_H_INNER_INTERSECT = b'\xce'.decode('ibm437')
    CHAR_H_INNER_VERTICAL = b'\xba'.decode('ibm437')
    CHAR_H_OUTER_LEFT_INTERSECT = b'\xcc'.decode('ibm437')
    CHAR_H_OUTER_LEFT_VERTICAL = b'\xba'.decode('ibm437')
    CHAR_H_OUTER_RIGHT_INTERSECT = b'\xb9'.decode('ibm437')
    CHAR_H_OUTER_RIGHT_VERTICAL = b'\xba'.decode('ibm437')
    CHAR_INNER_HORIZONTAL = b'\xcd'.decode('ibm437')
    CHAR_INNER_INTERSECT = b'\xce'.decode('ibm437')
    CHAR_INNER_VERTICAL = b'\xba'.decode('ibm437')
    CHAR_OUTER_BOTTOM_HORIZONTAL = b'\xcd'.decode('ibm437')
    CHAR_OUTER_BOTTOM_INTERSECT = b'\xca'.decode('ibm437')
    CHAR_OUTER_BOTTOM_LEFT = b'\xc8'.decode('ibm437')
    CHAR_OUTER_BOTTOM_RIGHT = b'\xbc'.decode('ibm437')
    CHAR_OUTER_LEFT_INTERSECT = b'\xcc'.decode('ibm437')
    CHAR_OUTER_LEFT_VERTICAL = b'\xba'.decode('ibm437')
    CHAR_OUTER_RIGHT_INTERSECT = b'\xb9'.decode('ibm437')
    CHAR_OUTER_RIGHT_VERTICAL = b'\xba'.decode('ibm437')
    CHAR_OUTER_TOP_HORIZONTAL = b'\xcd'.decode('ibm437')
    CHAR_OUTER_TOP_INTERSECT = b'\xcb'.decode('ibm437')
    CHAR_OUTER_TOP_LEFT = b'\xc9'.decode('ibm437')
    CHAR_OUTER_TOP_RIGHT = b'\xbb'.decode('ibm437')


class SingleTable(WindowsTable if IS_WINDOWS else UnixTable):
    """Cross-platform table with single-line box-drawing characters."""

    pass


class DoubleTable(WindowsTableDouble):
    """Cross-platform table with box-drawing characters. On Windows it's double borders, on Linux/OSX it's unicode."""

    pass
